module ProxyTester
  module Actions
    class InitializeApplication

      private

      attr_reader :config, :options

      public

      def initialize(options = {}, config = ProxyTester.config)
        @options = options
        @config  = config
      end

      def run
        create_test_cases_directory if options[:create_test_cases_directory]
        create_user_file if options[:create_user_file]
        create_config_file if options[:create_config_file]

        pre_seed if options[:pre_seed]

        show_example_config
      end

      private

      def create_test_cases_directory
        ProxyTester.ui_logger.info "Creating test case directory : #{config.test_cases_directory}"
        Actions::CreateDirectory.new(config.test_cases_directory, force: options[:force]).run
      end

      def create_config_file
        ProxyTester.ui_logger.info "Creating config file at \"#{config.config_file}\"."
        Actions::CreateFile.new(:'example-config', config.config_file, Data.new(config), force: options[:force], create_directories: true).run
      end

      def create_user_file
        ProxyTester.ui_logger.info "Creating user file at \"#{config.user_file}\"."

        data = OpenStruct.new(
          users: [
            {
              name: 'user1',
              password: 'password1',
              description: 'description',
            },
            {
              name: 'user2',
              password: 'password2',
              description: 'description',
            },
          ]
        )

        Actions::CreateFile.new(:'example-user_file', config.user_file, Data.new(data), force: options[:force], create_directories: true).run
      end

      def show_example_config
        ProxyTester.ui_logger.info "Showing the configuration of proxy_tester on your system."
        Actions::CreateOutput.new(:'example-config', $stdout, Data.new(config)).run
      end

      def pre_seed
        ProxyTester.ui_logger.info "Adding examples to at #{config.examples_directory}"
        Actions::AddExamplesToTestCasesDirectory.new(config.examples_directory, force: options[:force]).run
      end
    end
  end
end
