# encoding: utf-8
module ProxyTester
  class CapybaraProxyPac
    private

    attr_reader :content

    public

    attr_accessor :client_ip, :time, :pac_file, :url, :result

    def host
      result.proxy
    end

    def port
      result.proxy_port
    end

    def blank?
      host.blank? or port.blank?
    end

    def direct?
      result.request_type == 'DIRECT'
    end

    def verbatim
      result.verbatim
    end

    def pac_file=(source)
      uri = Addressable::URI.heuristic_parse(source)

      if uri.host.blank?
        @content = File.read(uri.path)
        @pac_file = uri.path
      else
        @content = open(uri, { proxy: false }).string
        @pac_file = uri.to_s
      end

      @content
    rescue Errno::ENOENT
      raise Exceptions::PacFileNotFound, JSON.dump(file: source)
    end

    private

    def result
      return PacResult.new if content.blank?
      return PacResult.new if url.blank?

      env_hash = {}
      env_hash[:client_ip] = client_ip if client_ip
      env_hash[:time]      = time if time

      parser = ProxyPacRb::Parser.new(ProxyPacRb::Environment.new(env_hash))

      file = parser.source(content)
      result = PacResult.new(file.find(url))

      raise Exceptions::PacResultResult, JSON.dump(file: pac_file, result: result) if result.blank?

      result
    end
  end
end
