# encoding
module ProxyTester
  module Cli
    class Main < Thor
      class_option :config_file, type: :string, desc: 'Config file'
      class_option :log_level, type: :string, desc: 'Log level for ui logging'
      class_option :debug_mode, type: :boolean, desc: 'Run application in debug mode'

      desc 'test', 'Run tests'
      subcommand 'test', Cli::Test

      desc 'add SOURCE', 'Add new testcases by cloning them via vcs'
      option :force, type: :boolean, default: false, desc: 'Overwrite existing files?'
      option :test_cases_directory, type: :string, desc: 'Directory with test cases'
      def add(source)
        ProxyTester.config = ProxyTester::Config.new(options[:config_file]) if options[:config_file]

        ProxyTester.config.log_level            = options[:log_level] if options[:log_level]
        ProxyTester.config.debug_mode           = options[:debug_mode] if options[:debug_mode]
        ProxyTester.config.test_cases_directory = options[:test_cases_directory] if options[:test_cases_directory]
        ProxyTester.config.lock

        ProxyTester.ui_logger.level = ProxyTester.config.log_level
        ProxyTester.enable_debug_mode if ProxyTester.config.debug_mode

        ProxyTester.ui_logger.debug('Options: ' + options.to_s)
        ProxyTester.ui_logger.debug("Config:\n" + ProxyTester.config.to_s)

        Actions::AddTestCase.new(RemoteRepository.new(source), ProxyTester.config.test_cases_directory, force: options[:force]).run
      end

      desc 'init', 'Create files/directories to use proxy_tester'
      option :force, type: :boolean, default: false, desc: 'Overwrite existing files?'
      option :pre_seed, type: :boolean, default: false, desc: 'Add examples to test cases directory'
      option :test_cases_directory, type: :string, desc: 'Directory with test cases'
      option :user_file, type: :string, desc: 'Path to user file'

      option :create_config_file, type: :boolean, desc: 'Create config file', default: true
      option :create_user_file, type: :boolean, desc: 'Create user file', default: true
      option :create_test_cases_directory, type: :boolean, desc: 'Create directory to store test cases', default: true
      def init
        ProxyTester.config = ProxyTester::Config.new(options[:config_file]) if options[:config_file]

        ProxyTester.config.log_level            = options[:log_level] if options[:log_level]
        ProxyTester.config.debug_mode           = options[:debug_mode] if options[:debug_mode]
        ProxyTester.config.test_cases_directory = options[:test_cases_directory] if options[:test_cases_directory]
        ProxyTester.config.user_file            = options[:user_file] if options[:user_file]
        ProxyTester.config.lock

        ProxyTester.ui_logger.level = ProxyTester.config.log_level
        ProxyTester.enable_debug_mode if ProxyTester.config.debug_mode

        ProxyTester.ui_logger.debug('Options: ' + options.to_s)
        ProxyTester.ui_logger.debug("Config:\n" + ProxyTester.config.to_s)

        Actions::InitializeApplication.new(
          force: options[:force], 
          pre_seed: options[:pre_seed], 
          create_config_file: options[:create_config_file],
          create_user_file: options[:create_user_file],
          create_test_cases_directory: options[:create_test_cases_directory],
        ).run
      end
    end
  end
end
