# encoding: utf-8
module ProxyTester
  # Exceptions
  module Exceptions
    # my standard error
    if RUBY_VERSION < '2.1'
      class MyStandardError < StandardError
        attr_reader :cause

        def initialize(msg, cause = $!)
          super(msg)
          @cause = cause;
        end
      end
    else
      class MyStandardError < StandardError; end
    end

    # raised if user error occurred
    class UserError < MyStandardError; end

    # raised if an internal error occurred
    class InternalError < MyStandardError; end

    # raised if an internal error occurred
    class DefaultError < MyStandardError; end

    # raised if config file does not exist
    class ConfigFileNotReadable < UserError; end

    # raised if one tries to access a lockec config 
    class ConfigLocked < UserError; end

    # raise if there are template syntax errrors
    class ErbTemplateHasSyntaxErrors < InternalError; end

    # raised if Template does not exist
    class ErbTemplateIsUnknown < InternalError; end

    # raised if commit does not exist
    class CommitDoesNotExist < InternalError; end

    # raised if repository does not exist
    class RepositoryDoesNotExist < UserError; end

    # raised if repository is not a supported vcs repository
    class RepositoryInvalid < UserError; end

    # raised if proxy user does not exist
    class ProxyUserInvalid < UserError; end

    # raised if proxy user record is invalid (missing name, password)
    class UserRecordInvalid < UserError; end

    # raised if user file not found 
    class UserFileNotFound < UserError; end

    # raised if user file not found 
    class SyntaxInvalid < UserError; end

    # raised if pac file not found 
    class PacFileNotFound < UserError; end

    # raised if pac result is invalid
    class PacResultInvalid < UserError; end

    # raised if pac result is invalid
    class FetchUrlTimeout < UserError; end

    # raised if chosen proxy type is invalid
    class ProxyTypeInvalid < UserError; end

    # raised if chosen proxy connection string is invalid
    class ProxyConnectionStringInvalid< UserError; end

    # raised if program was killed with SIGTERM
    class SIGTERM < UserError; end

    # raised if proxy is not reachable
    class ProxyNotReachable < UserError; end
  end
end
