# encoding: utf-8
module ProxyTester
  @environment = ProxyTester::Environment.new
  @config      = ProxyTester::Config.new
  @ui_logger   = ProxyTester::UiLogger.new
  @session     = ProxyTester::DatabaseSession.new

  class << self
    attr_accessor :session, :environment, :config, :ui_logger

    def root_path
      ::File.expand_path('../../..', __FILE__)
    end

    def enable_debug_mode
      ProxyTester.ui_logger.info "Activating debug mode."

      require 'pry'
      require 'byebug'
    rescue LoadError
      ProxyTester.ui_logger.error "You tried to enable debug-mode, but either 'pry'- or 'debugger'-gem are not installed. Please fix that before using the debug-switch again."
    end

    def configure_i18n
      I18n::Backend::Simple.send(:include, I18n::Backend::Fallbacks)
      I18n.load_path = Dir[::File.join(ProxyTester.root_path, 'lib', 'proxy_tester', 'locales', '*.yml')]
      I18n.backend.load_translations
      I18n.enforce_available_locales = true
    end

    def load_user_database
      User.load_from(UserDatabase.new)
    end

    def clear_environment
      %w{
      http_proxy
      https_proxy
      HTTP_PROXY
      HTTPS_PROXY
      }.each { |var| ENV.delete(var) }
    end
  end
end

ProxyTester.configure_i18n
