# encoding: utf-8
module ProxyTester
  class RemoteRepository

    private

    attr_reader :repository

    public

    # Create Remote Repository
    def initialize(repository)
      @repository = parse(repository)
    end

    # source path of repository
    #
    # @return [String]
    #   base path to repository
    #
    # @example
    #   http://githost/repo.git => repo
    #   user@host:/githost/repo.git => repo
    def base
      ::File.basename(repository.path, '.*')
    end

    # source path of repository
    #
    # @return [String]
    #   path to repository
    def source
      repository.to_s
    end

    private

    def parse(repo_string)
      repo_string = ::File.expand_path(repo_string) if repo_string =~ %r{^(?:[^/@]+/)*[^/]+$}

      repo = Addressable::URI.heuristic_parse(repo_string)

      if repo.scheme.blank?
        repo.scheme    = 'file'
        repo.authority = ''
        repo.path      = repo_string
      elsif repo.scheme == 'ssh'
        repo.host = repo.host + ':'
      end

      repo
    end

  end
end
