# encoding: utf-8

RSpec::Matchers.define :reference_domains do |expected|
  match do |actual|
    domains_eq? actual, expected
  end

  failure_message do |actual|
    expected_domains = format_domains(expected)
    actual_domains   = format_domains(extract_domains(actual))

    "expected that \"#{actual.current_url}\" references:\n#{expected_domains}\n\nBut it references:\n#{actual_domains}\n"
  end

  failure_message_when_negated do |actual|
    expected_domains = format_domains(expected)
    actual_domains   = format_domains(extract_domains(actual))

    "expected that \"#{actual.current_url}\" not references:\n#{expected_domains}\n\nBut it references:\n#{actual_domains}\n"
  end

  description do
    "to reference domains:\n#{format_domains(expected)}"
  end

  def domains_eq?(actual, expected)
    (extract_domains(actual) - expected).blank?
  end

  def extract_domains(page)
    page.driver.network_traffic.collect { |o| Addressable::URI.parse(o.url).host }.uniq
  end

  def format_domains(domains)
    domains.sort.collect { |d| format "* %s", d}.join("\n")
  end
end
