# encoding: utf-8
require 'rspec'
require 'bundler'

module ProxyTester
  class RspecRunner
    private 

    attr_reader :test_cases_directory, :tags

    public

    def initialize(options = {})
      @test_cases_directory = options.fetch(:test_cases_directory, ProxyTester.config.test_cases_directory)
      @tags                 = options.fetch(:tags, [])
    end

    def run
      ProxyTester.load_user_database
      ProxyTester.clear_environment

      $LOAD_PATH << test_cases_directory
      test_cases.each do |t|
        Dir.chdir t do
          Bundler.setup
          RSpec::Core::Runner.run(arguments)
        end
      end
    end

    def to_s
      arguments.join(" ")
    end

    private

    def test_cases
      ::Dir.glob(::File.join(test_cases_directory, '*')).keep_if { |d| FileTest.directory? d }
    end

    def arguments
      options = []
      options = options + ['--require', 'rspec/legacy_formatters' ]
      options << '--color'
      options = options + ['--format', 'Fuubar' ]
      options = options + ['--order', 'rand' ]

      Array(tags).each { |t| options << '--tag'; options << t }

      options << '.'

      options
    end

  end
end
